// library useful for development
const passport = require("passport");
const jwt = require("jsonwebtoken");
const {
  documentSuccessfulLogin,
} = require("../../service/activityLoggerService");

// function to handle the login process
const casLogin = async function (req, res, next) {
  console.log("Beginning log in....");

  // hard-coded for now.
  const JWT_SECRET = "secret_key";

  // if on localhost, accept the authentication and give a key
  console.log("user: ", process.env.DB_USER);

  if (process.env.DB_HOST === "mysql") {
    // document a successful login
    try {
      const timestamp = new Date(); // Current timestamp
      await documentSuccessfulLogin(1, timestamp); // userID = 1 for now?
    } catch (error) {
      console.error("Error logging the successful login: ", error);
    }

    // payload for the JWT token
    const payload = {
      id: 1,
      role: "admin",
      username: "localhostuser",
      peopleId: 1,
    };

    // sign the token with secret key
    const token = jwt.sign(payload, JWT_SECRET, { expiresIn: "1h" });

    // Set the token as an HTTP-only cookie
    res.cookie("auth_token", token, {
      maxAge: 3600000, // 1 hour in milliseconds
    });

    console.log("sent token for use in localhost");

    console.log("Frontend: ", process.env.FRONTEND_URL);

    res.redirect(`${process.env.FRONTEND_URL}/home`);
    return;
  }

  // call the authenticate function with CAS
  passport.authenticate("cas", async function (err, user, info) {
    if (err) {
      console.log("found error during authentication: ", err);
      return next(err);
    }

    // return 401 if user not found
    if (!user) {
      console.log("User not found: ", user);
      return res.status(401).json({ message: "Authentication failed." });
    }

    console.log("Found user");

    // create a user object and redirect user
    // to the root route
    req.logIn(user, async function (err) {
      console.log("Executing log in.");
      if (err) {
        return next(err);
      }

      console.log("User: ", user);

      // payload for the JWT token
      const payload = {
        id: user.userID,
        role: user.permissionLevel || "guest",
        username: user.username,
        peopleId: user.peopleId,
      };

      // sign the token with secret key
      const token = jwt.sign(payload, JWT_SECRET, { expiresIn: "1h" });

      res.cookie("auth_token", token, {
        maxAge: 3600000, // 1 hour in milliseconds
      });

      // document a successful login
      try {
        const timestamp = new Date(); // Current timestamp
        await documentSuccessfulLogin(user.userID, timestamp);
      } catch (error) {
        console.error("Error logging the successful login: ", error);
      }

      // Send the JWT token to the frontend
      console.log("sending JWT To: ", process.env.FRONTEND_URL);
      res.redirect(`${process.env.FRONTEND_URL}/home`);

      console.log("Successfully authenticated and generated token.");
    });
  })(req, res, next);
};

module.exports = casLogin;
