const {
  addAddress,
  getAddressesForPerson,
  deleteAddressById,
  updateAddress,
} = require("../repository/addressRepository");
const {
  addContact,
  getContactsForPerson,
  updateContact,
  deleteContactById,
} = require("../repository/contactRepository");
const {
  addDegree,
  getDegreeIdsForPerson,
  deleteDegreeById,
  updateDegree,
  getDegreesForPerson,
} = require("../repository/degreeRepository");
const {
  addPerson,
  searchPeopleByName,
  deletePersonById,
  updatePerson,
  getPersonById,
  search,
} = require("../repository/peopleRepository");
const {
  addPeopleInvolvement,
  getInvolvementType,
  getInvolvement,
  getInvolvementsForPerson,
} = require("../repository/involvementRepository");
const { upsertUser } = require("../repository/userRepository");
const xlsx = require('xlsx');

/*
Function to handle the logic for reading in a file and creating all people listed in file.
*/
async function readFile(file) {
  console.log(file.name);
  try {
      const myFile = xlsx.readFile(file); // Read the file
      console.log("hereA");
      // Loop through all sheets
      myFile.SheetNames.forEach(sheetName => {
          const worksheet = myFile.Sheets[sheetName];
          console.log("hereB");
          // Convert the sheet data to JSON
          let jsonData = xlsx.utils.sheet_to_json(worksheet);

          // Trim headers and convert to lowercase
          jsonData = jsonData.map(row => {
              const newRow = {};
              for (const key in row) {
                  // Trim the key and convert it to lowercase
                  const trimmedKey = key.trim().toLowerCase();
                  newRow[trimmedKey] = row[key]; // Keep the original value
              }
              // console.log(newRow);
              return newRow;
          });
          console.log("hereC");
          jsonData.forEach(row => {
              // Access data using header names
              const preferredYr = row['preferred class year']; // Will the headers always be the same?? -- leading and trailing space and caps dont matter
              const id = row['id'];
              const notes = row['notes'];
              const lastName = row['last name'];
              const firstName = row['first name'];
              const nickname = row['nickname'];
              const maidenName = row['maiden name']
              const middleName = row['middle name']
              const gender = row['gender']
              const address = row['address']
              const address2 = row['address 2']
              const zip = row['zip']
              const city = row['city']
              const state = row['state']
              const chapter = row['chapter']
              const phone = row['phone']
              const phone2 = row['phone 2']
              const doesNotGetEmail = row['does not receive alumni email']
              const email = row['email']
              const email2 = row['email 2']
              const email3 = row['email 3']
              const email4 = row['email 4']
              const preferredCollege = row['preferred college']
              const degree1 = row['degree 1']
              const department1 = row['department 1']
              const college1 = row['college 1']
              const degree2 = row['degree 2']
              const department2 = row['department 2']
              const college2 = row['college 2']
              const involvements = {}
      
              for (let i = 0; i < 15; i++) {
                  let num = i+1
                  let value = "involvement " + num
                  involvements[i] = row[value]
              }
              console.log("middle")
              // console.log(`PreferredYr: ${preferredYr}, lastname: ${lastName}`); //to test
      
              //now create new person with these values and call createPerson(person)
      
              const person = {
                personInfo: {
                    firstName: firstName || "", // Default to empty string if not available
                    lastName: lastName || "",
                    middleName: middleName || "",
                    maidenName: maidenName || "",
                    suffix: "", // Default to empty string
                    nickName: nickname || "",
                    techAlumniChapter: chapter || "", // Default if not provided
                    classYear: preferredYr || null, // Default to null if not provided
                    gradYear: gradYear || null, // Set to null if not available
                    gradSemester: gradSemester || "", // Default to empty string
                    gender: gender || "", // Default to empty string
                },
                degrees: [
                    {
                        degreeType: degree1 || "", // Set to default if not available
                        degreeYear: gradYear || null, // Default to null if not provided
                        degreeDescription: degreeDescription || "", // Default to empty string if not provided
                        degreeDepartment: department1 || "", // Default to empty string if not provided
                        degreeCollege: college1 || "" // Default to empty string if not provided
                    }
                ],
                addresses: [
                    {
                        address1: address || "", // Default to empty string
                        address2: address2 || "", // Default to empty string
                        city: city || "",
                        state: state || "",
                        country: "USA", // Default to "USA" if not provided
                        zipCode: zip || null, // Default to null if not provided
                        preferredAddress: true // Assuming it is preferred by default
                    }
                ],
                contacts: [
                    {
                        contactNumber: phone || "",
                        contactType: "phone", // Assuming 'phone' for all contact types
                        preferredContact: true // Assuming this contact is preferred by default
                    }
                ],
                involvements: [
                    involvements[1] || "", // Assuming you want each "involvement" as an array of strings
                    involvements[2] || "",
                    involvements[3] || "",
                    involvements[4] || "",
                    involvements[5] || "",
                    involvements[6] || "",
                    involvements[7] || "",
                    involvements[8] || "",
                    involvements[9] || "",
                    involvements[10] || "",
                    involvements[11] || "",
                    involvements[12] || "",
                    involvements[13] || "",
                    involvements[14] || ""
                ].filter(value => value !== ""), // Remove empty strings if no involvements
                userInfo: null // Default to null as required
            };
              console.log("before create");
              // console.log(person);
      
              createPerson(person);  
                          
          });
      });

} catch (error) {
  console.log("ERROR")
  console.error(`Error reading file: ${error.message}`);
}
}

/*
Function to handle the logic for creating a person.
*/
async function createPerson(person) {
  // extract the parts of a person
  const { personInfo, degrees, addresses, contacts, involvements, userInfo } =
    person;

  console.log("Adding person....");
  // insert the personInfo into people table
  const newPersonID = await addPerson(personInfo);
  console.log("Added person ID: ", newPersonID);

  // insert the degree information into the database
  for (const degree of degrees) {
    const addedDegree = await addDegree(degree, newPersonID);
    console.log("added degree: ", addedDegree);
  }

  // insert the address information into the database
  for (const address of addresses) {
    const addedAddress = await addAddress(address, newPersonID);
    console.log("added address: ", addedAddress);
  }

  // insert the contact information into the database
  for (const contact of contacts) {
    const addedContact = await addContact(contact, newPersonID);
    console.log("added contact: ", addedContact);
  }

  // insert the involvement information into the database
  for (const involvement of involvements) {
    // retrieve involvement Type
    const involvementType = await getInvolvementType(involvement);

    if (involvementType) {
      // get the invovelment id and link to person
      const involvement = await getInvolvement(
        involvementType.involvementTypeId
      );
      if (involvement) {
        await addPeopleInvolvement(newPersonID, involvement.involvementId);
      } else {
        console.log("invovlement not found");
      }
    } else {
      console.log("involvement does not exist: ", involvement);
    }
  }

  // add the user info
  if (userInfo) {
    // get the pid
    const pid = userInfo.pid;
    if (pid) {
      try {
        await upsertUser(
          userInfo.pid,
          userInfo.roleId,
          userInfo.info,
          newPersonID
        );
      } catch (error) {
        console.log("error upserting user.");
      }
    } else {
      console.log("No pid found");
    }
  }

  // TODO: log database change here

  return newPersonID;
}

/*
Function to call repository and get people with a 
gicen first and last name.
*/
async function findByName(firstName, lastName) {
  // call repository function, null will be returned if no people are found
  const peopleFound = await searchPeopleByName(firstName, lastName);
  return peopleFound;
}

/*
Function to delete person by their id.
*/
async function deleteById(peopleId) {
  // delete the person
  const personDeleted = await deletePersonById(peopleId);

  if (personDeleted) {
    return true;
  }

  return false;
}

/*
Function to update all of a person with the given id's information.
*/
async function updatePersonById(personId, person) {
  console.log("Person: ", person);

  const { personalInformation, degreeInformation, addressInformation, contactInformation, involvmentInformation } = person;

  console.log("Updating person...");
  console.log(personalInformation)

  // update personInfo
  const updatedPersonInfo = await updatePerson(personId, personalInformation);
  if (updatedPersonInfo == false) {
    return null;
  }

  console.log("updated person info");

  // update degrees
  if (degreeInformation) {
    await updateDegreesForPerson(personId, degreeInformation);
  }

  // update address info
  if (addressInformation) {
    await updateAddressesForPerson(personId, addressInformation);
  }

  // update contact info
  if (contactInformation) {
    await updateContactsForPerson(personId, contactInformation);
  }

  if (involvmentInformation) {
    await updateInvolvementsForPerson(personId, involvmentInformation);
  }

  return true;
}

/*
Function to get all the data that is associated with a single 
person.
*/
async function getPersonFullDetails(personId) {
  // get the personal details
  const personalInformation = await getPersonById(personId);

  // get the degree information
  const degreeInformation = await getDegreesForPerson(personId);

  // get the address information
  const addressInformation = await getAddressesForPerson(personId);

  // get the contact information
  const contactInformation = await getContactsForPerson(personId);

  // get the involvement information (none for now)
  const involvmentInformation = await getInvolvementsForPerson(personId);

  // return all of the data
  return {
    personalInformation,
    degreeInformation,
    addressInformation,
    contactInformation,
    involvmentInformation,
  };
}

/*
Helper function to update the degree information for a person.
*/
async function updateDegreesForPerson(personId, degrees) {
  // Get the person's current degrees
  const currentDegreeIds = await getDegreeIdsForPerson(personId);
  let updatedDegreeIds = [];

  // Add/update the given degrees
  for (const degree of degrees) {
    const peopleDegreeId = degree.peopleDegreeId || null;

    // degree does not previously have a peopleDegreeId -> this means its new (add degree)
    if (!peopleDegreeId) {
      const addedDegree = await addDegree(degree, personId);
      console.log("Added new degree: ", addedDegree);
    } else {
      // previous peopleDegreeId exists, update the degree
      const updatedDegree = await updateDegree(
        peopleDegreeId,
        personId,
        degree
      );
      console.log("Updated degree: ", updatedDegree);
      if (updatedDegree == true) {
        updatedDegreeIds.push(peopleDegreeId);
      }
    }
  }

  // Delete degrees not included in the update
  for (const degreeId of currentDegreeIds) {
    if (!updatedDegreeIds.includes(degreeId)) {
      await deleteDegreeById(degreeId);
      console.log("Deleted degree: ", degreeId);
    }
  }
}

/*
Helper function to update addresses for a person.
*/
async function updateAddressesForPerson(personId, addresses) {
  // Get the current addresses
  const currentAddresses = await getAddressesForPerson(personId);
  let updatedAddressIds = [];

  for (const address of addresses) {
    const addressId = address.addressId || null;

    if (!addressId) {
      const addedAddress = await addAddress(address, personId);
      console.log("Added new address: ", addedAddress);
    } else {
      const updatedAddress = await updateAddress(addressId, personId, address);
      if (updatedAddress != null) {
        console.log("Updated address: ", updatedAddress);
        updatedAddressIds.push(addressId);
      } else {
        console.log("Error updating adddress");
      }
    }
  }

  // Delete addresses not included in the update
  for (const currentAddress of currentAddresses) {
    if (!updatedAddressIds.includes(currentAddress.addressId)) {
      await deleteAddressById(currentAddress.addressId);
      console.log("Deleted address: ", currentAddress.addressId);
    }
  }
}

/*
Helper function to update contacts for a person.
*/
async function updateContactsForPerson(personId, contacts) {
  // Get the current contacts
  const currentContacts = await getContactsForPerson(personId);
  let updatedContactIds = [];

  for (const contact of contacts) {
    const contactId = contact.peopleContactId || null;

    if (!contactId) {
      const addedContact = await addContact(contact, personId);
      console.log("Added new contact: ", addedContact);
    } else {
      const updatedContact = await updateContact(contactId, personId, contact);
      console.log("Updated contact: ", updatedContact);
      if (updatedContact == true) {
        updatedContactIds.push(contactId);
      }
    }
  }

  // Delete contacts not included in the update
  for (const currentContact of currentContacts) {
    if (!updatedContactIds.includes(currentContact.peopleContactId)) {
      // TODO: delete contact by id
      let success = await deleteContactById(currentContact.peopleContactId);
      if (success == true) {
        console.log("Deleted contact: ", currentContact.peopleContactId);
      } else {
        console.log("Error deleting contact: ", currentContact.peopleContactId);
      }
    }
  }
}

async function updateInvolvementsForPerson(personId, involevements) {
  // Step 1: Retrieve the current involvements for the person
  const currentInvolvements = await getInvolvementsForPerson(personId);

  console.log("current involvements: ", currentInvolvements);

  const newInvolvementIds = [];
  for (const involvement of involevements) {
    // get the involvement type for each invovlement
    const involvementType = await getInvolvementType(involvement);

    if (involvementType) {
      // get the invovelment id and link to person
      const involvement = await getInvolvement(
        involvementType.involvementTypeId
      );
      if (involvement) {
        console.log("invovlement id: ", involvement.involvementId);
        newInvolvementIds.push(involvement.involvementId);

        if (!currentInvolvements.includes(involvement)) {
          await addPeopleInvolvement(personId, involvement.involvementId);
        } else {
          console.log("involvement already here!");
        }
      } else {
        console.log("invovlement not found");
      }
    }
  }
}

async function generalSearch(filters) {
  return search(filters);
}

module.exports = {
  readFile,
  createPerson,
  findByName,
  deleteById,
  updatePersonById,
  getPersonFullDetails,
  generalSearch,
};
